'use strict';
/** @jsx preact.h */

var preact = require('preact');

var { render, h, Component } = preact;
const {Row} = require('./Row');
const {Store} = require('./Store');

var startTime;
var lastMeasure;
var startMeasure = function(name) {
  // console.timeStamp(name);
  startTime = performance.now();
  lastMeasure = name;
};
var stopMeasure = function() {
  var last = lastMeasure;
  if (lastMeasure) {
    window.setTimeout(function() {
      lastMeasure = null;
      var stop = performance.now();
      var duration = 0;
      console.log(last + ' took ' + (stop - startTime));
    }, 0);
  }
};

export class Main extends Component {
  constructor(props) {
    super(props);
    this.state = {store: new Store()};
    this.select = this.select.bind(this);
    this.delete = this.delete.bind(this);
    this.add = this.add.bind(this);
    this.run = this.run.bind(this);
    this.update = this.update.bind(this);
    this.runLots = this.runLots.bind(this);
    this.clear = this.clear.bind(this);
    this.swapRows = this.swapRows.bind(this);
    this.start = 0;
    this.length = 0;

    window.app = this;
  }
  printDuration() {
    stopMeasure();
  }
  componentDidUpdate() {
    this.printDuration();
  }
  componentDidMount() {
    this.printDuration();
  }
  run() {
    startMeasure('run');
    this.state.store.run();
    this.setState({store: this.state.store});
  }
  add() {
    startMeasure('add');
    this.state.store.add();
    this.setState({store: this.state.store});
  }
  update() {
    startMeasure('update');
    this.state.store.update();
    this.setState({store: this.state.store});
  }
  select(id) {
    startMeasure('select');
    this.state.store.select(id);
    this.setState({store: this.state.store});
  }
  delete(id) {
    startMeasure('delete');
    this.state.store.delete(id);
    this.setState({store: this.state.store});
  }
  runLots() {
    startMeasure('runLots');
    this.state.store.runLots();
    this.setState({store: this.state.store});
  }
  clear() {
    startMeasure('clear');
    this.state.store.clear();
    this.setState({store: this.state.store});
  }
  swapRows() {
    startMeasure('swapRows');
    this.state.store.swapRows();
    this.setState({store: this.state.store});
  }
  render() {
    let rows = this.state.store.data.map((d, i) => {
      return <Row key={d.id} data={d} onClick={this.select} onDelete={this.delete} styleClass={d.id === this.state.store.selected ? 'danger' : ''} />;
    });
    return (<div className="container">
      <div className="jumbotron">
        <div className="row">
          <div className="col-md-6">
            <h1>preact</h1>
          </div>
          <div className="col-md-6">
            <div className="row">
              <div className="col-sm-6 smallpad">
                <button type="button" className="btn btn-primary btn-block" id="run" onClick={this.run}>Create 1,000 rows</button>
              </div>
              <div className="col-sm-6 smallpad">
                <button type="button" className="btn btn-primary btn-block" id="runlots" onClick={this.runLots}>Create 10,000 rows</button>
              </div>
              <div className="col-sm-6 smallpad">
                <button type="button" className="btn btn-primary btn-block" id="add" onClick={this.add}>Append 1,000 rows</button>
              </div>
              <div className="col-sm-6 smallpad">
                <button type="button" className="btn btn-primary btn-block" id="update" onClick={this.update}>Update every 10th row</button>
              </div>
              <div className="col-sm-6 smallpad">
                <button type="button" className="btn btn-primary btn-block" id="clear" onClick={this.clear}>Clear</button>
              </div>
              <div className="col-sm-6 smallpad">
                <button type="button" className="btn btn-primary btn-block" id="swaprows" onClick={this.swapRows}>Swap Rows</button>
              </div>
            </div>
          </div>
        </div>
      </div>
      <table className="table table-hover table-striped test-data">
        <tbody>
          {rows}
        </tbody>
      </table>
      <span className="preloadicon glyphicon glyphicon-remove" aria-hidden="true" />
    </div>);
  }
}

render(<Main />, document.getElementById('main'));